#! /usr/bin/env python
#
# util_RandomizeOverlapOrder.py
#
# 
# GOAL
#   - take an overlap.xml.gz (SimInspiralTable), and randomize its order.  
#   - Important when merging files from many workers, to avoid accidentally using only the output from one of them.  Run after ligolw_add
#   - takes equal number from each file by default ... WARNING THIS MEANS WE CAN HAVE PATHOLOGICAL SIZE LIMITS


import sys
from optparse import OptionParser
import numpy as np
from ligo.lw import utils, table, lsctables, ligolw
try:
    import h5py
except:
    print(" - no h5py - ")

import RIFT.lalsimutils as lalsimutils

# Contenthandlers : argh
#   - http://software.ligo.org/docs/glue/
lsctables.use_in(ligolw.LIGOLWContentHandler)

optp = OptionParser()
optp.add_option("--fref",default=20,type=float,help="Reference frequency. Depending on approximant and age of implementation, may be ignored")
optp.add_option("--n-min",default=20,type=int,help="Minimum size of file to include")
optp.add_option("--output-file",default='merged_output',type=str,help="Merged output file")
opts, args = optp.parse_args()


P_list = []
P_list_list =[]
n_list = []
for fname in args:
    P_list_this_file = lalsimutils.xml_to_ChooseWaveformParams_array(fname)
    if len(P_list_this_file)<opts.n_min:
        continue
    P_list_list.append(P_list_this_file)
    n_list.append(len(P_list_this_file))
n_min = np.min(n_list)
for indx in np.arange(len(P_list_list)):
    indx_to_take = np.random.choice(np.arange(len(P_list_list[indx])),size=n_min)
    P_to_add = [P_list_list[indx][a] for a in indx_to_take]
    P_list += P_to_add

lalsimutils.ChooseWaveformParams_array_to_xml(P_list,opts.output_file)



#! /usr/bin/env python
#
# util_NRDumpDetectorResponse.py
#
# EXAMPLE
#    util_NRDumpDetectorResponse.py --inj inj.xml.gz --event 0 --group GROUP --param PARAM --ifo H
#
# Intended to reproduce events found by ILE.  Will use real sky locations, total masses, etc


import argparse
import numpy as np

import RIFT.lalsimutils as lalsimutils
import lalsimulation as lalsim
import lal
import sys

import NRWaveformCatalogManager3 as nrwf

parser = argparse.ArgumentParser()
parser.add_argument("--inj",default=None)
parser.add_argument("--event",default=None,type=int)
parser.add_argument("--group", default="Sequence-GT-Aligned-UnequalMass",help="inspiral XML file containing injection information.")
parser.add_argument("--param", default=(0.0, 2.),help="Parameter value")
parser.add_argument("--hybrid-use",action='store_true',help="Enable use of NR (or ROM!) hybrid, using --approx as the default approximant and with a frequency fmin")
parser.add_argument("--hybrid-method",default="taper_add",help="Hybridization method for NR (or ROM!).  Passed through to LALHybrid. pseudo_aligned_from22 will provide ad-hoc higher modes, if the early-time hybridization model only includes the 22 mode")
parser.add_argument("--no-memory",action='store_true')
parser.add_argument("--seglen",default=8,type=int)
parser.add_argument("--t-ref",default=None,type=float,help="Zero of time to use on exported data files. (default is to use the time in the XML")
parser.add_argument("--use-perturbative-extraction",default=False,action='store_true')  # this should be the default with ILE operation
parser.add_argument("--use-provided-strain",default=False,action='store_true')  # this should NEVER be used, provided for testing only
parser.add_argument("--l", default=2, type=int)
parser.add_argument("--rextr", default=None,type=int)
parser.add_argument("--verbose",action='store_true')
parser.add_argument("--approx",default='SEOBNRv2',help="Approximant to use for LAL comparisons. Can be EOBNRv2HM or SEOBNRv2")
parser.add_argument("--save-plots",default=False,action='store_true', help="Write plots to file (only useful for OSX, where interactive is default")
opts = parser.parse_args()

print(" WARNING: Use this package only when you record TIME INFORMATION (i.e., --maximization-only). ")

T_window = opts.seglen
deltaT = 1./4096

bNoInteractivePlots=True # default
fig_extension = '.png'
try:
    import matplotlib
    print(" Matplotlib backend ", matplotlib.get_backend())
    if matplotlib.get_backend() is 'MacOSX':
        if opts.save_plots:
            print("  OSX without interactive plots")
            bNoInteractivePlots=True
            fig_extension='.png'
        else:  #  Interactive plots
            print("  OSX with interactive plots")
            bNoInteractivePlots=False
    elif matplotlib.get_backend() is 'agg':
        fig_extension = '.png'
        bNoInteractivePlots=True
        print(" No OSX; no interactive plots ")
    else:
        print(" Unknown configuration ")
        fig_extension = '.png'
        bNoInteractivePlots =True
    from matplotlib import pyplot as plt
    bNoPlots=False
except:
    from matplotlib import pyplot as plt
    fig_extension = '.png'
    print(" - no matplotlib - ")
    bNoInteractivePlots = False
    bNoPlots = True

group = opts.group
if not nrwf.internal_ParametersAreExpressions[group]:
    param = opts.param
else:
    param = eval(str(opts.param))
if opts.verbose:
    print("Importing ", group, param)


## LOAD INJECTION FILE
#   - complete garbage as waveform entry, to make sure the code to load waveforms operates PERIOD
print(" Loading injection file ... ")
# force_waveform='TaylorT4threePointFivePN',,force_taper='TAPER_NONE'
P = lalsimutils.xml_to_ChooseWaveformParams_array(str(opts.inj))[opts.event]  # Load in the physical parameters of the inj

## LOAD WEIGHT REPORT (just in case)
# from ligo.lw import lsctables, utils, table
# samples = table.get_table(utils.load_filename(opts.inj), lsctables.SimInspiralTable.tableName)
# lnLmax = np.max([s.alpha1 for s in samples])
# def sorted_indexes(seq):
#     return [i for (v, i) in sorted((v, i) for (i, v) in enumerate(seq))]
# samples_indx_sorted =sorted_indexes( [np.exp(s.alpha-lnLmax)*s.alpha2/s.alpha3 for s in samples])
# print samples[opts.event]
# the_sample = samples[opts.event]
# print " Weight for this sample ", the_sample.alpha  + np.log(the_sample.alpha2/the+sample.alpha3)

## LOAD NR SIMULATION
print(" Loading NR simulation ... ")
no_memory = False
wfP = nrwf.WaveformModeCatalog(opts.group, param, clean_initial_transient=True,clean_final_decay=True, shift_by_extraction_radius=True, extraction_radius=opts.rextr,lmax=opts.l,align_at_peak_l2_m2_emission=True,perturbative_extraction=opts.use_perturbative_extraction,use_provided_strain=opts.use_provided_strain)
if opts.no_memory and wfP.P.SoftAlignedQ():
    print(" STRIPPING MEMORY")
    no_memory = True


## COPY INJECTION FILE CORE SETTINGS TO wfP.P: only mass and 7 extrinsic parameters matter for NR
#   - could use P.manual_copy, but want to be CAREFUL.
# total mass
wfP.P.m1 = P.m1
wfP.P.m2 =P.m2
# seven intrinsic parameters
wfP.P.dist = P.dist
wfP.P.incl = P.incl
wfP.P.phiref=P.phiref
wfP.P.theta = P.theta
wfP.P.phi = P.phi
wfP.P.psi = P.psi
wfP.P.tref = P.tref  # this is a GEOCENTRIC TIME for now
wfP.P.detector = 'H1'  # for now
wfP.P.radec =True
wfP.P.deltaF = 1./T_window  # 1
wfP.P.deltaT = deltaT

wfP.P.print_params()

data_dict_T ={}
fig_list = {}
indx = 0
t_ref = wfP.P.tref   # This SHOULD NOT BE USED, if you want to time-align your signal
if opts.t_ref:
    t_ref = float(opts.t_ref)
else:
    print(" Warning: no reference time, it will be very difficult to time-align your signals ")
for ifo in ['H1', 'L1', 'V1']:
    indx += 1
    fig_list[ifo] = indx
    plt.figure(indx)
    wfP.P.detector = ifo
    data_dict_T[ifo] = wfP.real_hoft(no_memory=opts.no_memory,hybrid_use=opts.hybrid_use,hybrid_method=opts.hybrid_method)
    tvals = data_dict_T[ifo].deltaT*np.arange(data_dict_T[ifo].data.length) + float(data_dict_T[ifo].epoch)
    det = lalsim.DetectorPrefixToLALDetector(ifo)
    print(ifo, " T_peak =", P.tref + lal.TimeDelayFromEarthCenter(det.location, P.phi, P.theta, P.tref) - t_ref)
    if opts.verbose:
        plt.plot(tvals - wfP.P.tref, data_dict_T[ifo].data.data)

    np.savetxt(ifo+"_nr_"+group+"_"+str(param)+"_event_"+str(opts.event)+".dat", np.array([tvals -t_ref, data_dict_T[ifo].data.data]).T)

if opts.verbose and not opts.save_plots:
    plt.show()
if opts.save_plots:
    for ifo in  fig_list:
        plt.figure(fig_list[ifo])
        plt.savefig("response-"+str(ifo)+group+"_"+str(param)+fig_extension)
        






#! /usr/bin/env python
#
# GOAL
#  - Load in LI posterior samples
#  - generate waveforms using all the real parameters, at H and L
#  - compute overlaps due to two approximations
#
# ALSO USEFUL FOR COMPARISON
#  util_TestPrecessingProperties.py   # plots h(t), h(f) for precessing binaries, to illustrate their similarity
#  util_NRCompareSimulations.py  # overlaps


import argparse
import sys
import numpy as np
import scipy
import RIFT.lalsimutils as lalsimutils
import lalsimulation as lalsim
import lalframe
import lal

try:
    from matplotlib import pyplot as plt
except:
    print(" - no plots - ")

parser = argparse.ArgumentParser()
parser.add_argument("--approx",default="SEOBNRv2")
parser.add_argument("--approx2",default="SpinTaylorT4")
parser.add_argument("--use-precessing",action='store_true')
parser.add_argument("--fname-lalinference",help="filename of posterior_samples.dat file [standard LI output], to overlay on corner plots")
#parser.add_argument("--fmin",type=float,default=None)
parser.add_argument("--fisher-psd",type=str,default="SimNoisePSDaLIGOZeroDetHighPower",help="psd name (attribute in lalsimulation).  SimNoisePSDiLIGOSRD, lalsim.SimNoisePSDaLIGOZeroDetHighPower, lalsimutils.Wrapper_AdvLIGOPsd, .SimNoisePSDiLIGOSRD... ")
parser.add_argument("--psd-file",  help="File name for PSD (assumed hanford). Overrides --fisher-psd if provided")
parser.add_argument("--srate",type=int,default=16384,help="Sampling rate")
parser.add_argument("--seglen", type=float,default=8., help="Default window size for processing.")
parser.add_argument("--fmax",default=1700,type=float)
parser.add_argument("--save-plots",action='store_true')
parser.add_argument("--verbose",action='store_true')
opts=  parser.parse_args()

# Handle PSD
# FIXME: Change to getattr call, instead of 'eval'
eff_fisher_psd = lalsim.SimNoisePSDiLIGOSRD
if not opts.psd_file:
    #eff_fisher_psd = eval(opts.fisher_psd)
    eff_fisher_psd = getattr(lalsim, opts.fisher_psd)   # --fisher-psd SimNoisePSDaLIGOZeroDetHighPower   now
    analyticPSD_Q=True
else:
    print(" Importing PSD file ", opts.psd_file)
    eff_fisher_psd = lalsimutils.load_resample_and_clean_psd(opts.psd_file, 'H1', 1./opts.seglen)
    analyticPSD_Q = False


print(" Loading samples ")
samples_in = np.genfromtxt(opts.fname_lalinference,names=True)
print(" Done loading samples ")
deltaT = 1./4096
T_window =int(opts.seglen)

# Create overlap 
P=lalsimutils.ChooseWaveformParams()
P.deltaT = deltaT
P.deltaF = 1./T_window
P.fmin = samples_in["flow"][0]
P.approx=lalsim.SpinTaylorT2
P.print_params()
print(lalsimutils.estimateDeltaF(P), P.deltaF)
if P.deltaF > lalsimutils.estimateDeltaF(P):
    sys.exit(0)
hfBase = lalsimutils.complex_hoff(P) # Does not matter what this is.
print(" Done generating waveform")
IP = lalsimutils.CreateCompatibleComplexOverlap(hfBase,analyticPSD_Q=analyticPSD_Q,psd=eff_fisher_psd,fMax=opts.fmax,interpolate_max=True)
print(" Done populating inner product")


# Similar code: convert_output_format_inference2ile
fac_reduce=1
for indx in np.arange(len(samples_in["m1"])):
#    print indx
    ###
    ###  Define the system
    ###
    m1 = samples_in["m1"][indx]*lal.MSUN_SI
    m2 = samples_in["m2"][indx]*lal.MSUN_SI
    d = samples_in["distance"][indx]*lal.PC_SI*1e6
    P = lalsimutils.ChooseWaveformParams(m1=m1,m2=m2,dist=d)
    P.taper =lalsim.SIM_INSPIRAL_TAPER_START
    P.radec=True
    P.theta = samples_in["dec"][indx]
    P.phi = samples_in["ra"][indx]
    if "time_maxl" in samples_in.dtype.names:
     P.time = samples_in["time_maxl"][fac_reduce*indx]
    else:
     P.time = samples_in["time"][indx]
    P.fmin = samples_in["flow"][indx]
    P.fref = samples_in["f_ref"][indx]  # a field now !  MAY NOT BE RELIABLE
    if "phi_orb" in samples_in.dtype.names:
     P.phiref = samples_in["phi_orb"][fac_reduce*indx]
    elif "phase_maxl" in samples_in.dtype.names:
     P.phiref =samples_in["phase_maxl"][fac_reduce*indx]
    elif "phase" in samples_in.dtype.names:
     P.phiref = samples_in["phase"][fac_reduce*indx]
    else:
     print(samples_in.dtype.names)
     P.phiref = 0  # does not actually matter
    P.approx = lalsim.GetApproximantFromString(opts.approx)
    if "phi_jl" in samples_in.dtype.names and 'theta1' in samples_in.dtype.names:
      P.init_via_system_frame( 
         thetaJN=samples_in["theta_jn"][fac_reduce*indx],
         phiJL=samples_in["phi_jl"][fac_reduce*indx],
         theta1=samples_in["tilt1"][fac_reduce*indx],
         theta2=samples_in["tilt2"][fac_reduce*indx],
         phi12=samples_in["phi12"][fac_reduce*indx],
         chi1=samples_in["a1"][fac_reduce*indx],
         chi2=samples_in["a2"][fac_reduce*indx],
         psiJ=samples_in["psi"][fac_reduce*indx]   # THIS IS NOT BEING SET CONSISTENTLY...but we marginalize over it, so that's ok
         )
    elif P.approx == lalsim.SEOBNRv2 or  P.approx == lalsimutils.lalSEOBv4 or P.approx == lalsimutils.lalIMRPhenomD or P.approx == lalsim.IMRPhenomC:
        # Aligned spin model
        P.s1z = samples_in["a1z"][fac_reduce*indx]
        P.s2z = samples_in["a2z"][fac_reduce*indx]
        P.psi = samples_in["psi"][fac_reduce*indx]
        if "theta_jn" in samples_in.dtype.names:
                P.incl = samples_in["theta_jn"][fac_reduce*indx]

    elif 'theta1' in 'theta1' in samples_in.dtype.names:
      P.init_via_system_frame( 
         thetaJN=samples_in["theta_jn"][fac_reduce*indx],
         phiJL=0, # does not matter
         theta1=samples_in["theta1"][fac_reduce*indx],
         theta2=samples_in["theta2"][fac_reduce*indx],
         phi12=samples_in["phi12"][fac_reduce*indx],
         chi1=samples_in["a1"][fac_reduce*indx],
         chi2=samples_in["a2"][fac_reduce*indx],
         psiJ=samples_in["psi"][fac_reduce*indx]   # THIS IS NOT BEING SET CONSISTENTLY...but we marginalize over it, so that's ok
         )
    else:
        print(" Don't know how to handle this orientation for", opts.approx)

      ###
      ### Create two approximants
      ###
    P.deltaT = deltaT
    P.deltaF = 1./T_window
    if opts.verbose:
        P.print_params()

    P.approx = lalsim.GetApproximantFromString(opts.approx)
    P.fmax = opts.fmax
    if P.approx == lalsim.SEOBNRv2 or P.approx == lalsim.TaylorF2 or P.approx == lalsim.IMRPhenomC or P.approx==lalsim.IMRPhenomD :
        P.s1x = 0
        P.s2x = 0
        P.s1y=0
        P.s2y=0
    hF_1 = lalsimutils.complex_hoff(P)

    P.approx = lalsim.GetApproximantFromString(opts.approx2)
    hF_2 = lalsimutils.complex_hoff(P)

    nm_1 = IP.norm(hF_1)
    nm_2 = IP.norm(hF_2)
    match = IP.ip(hF_1,hF_2)/nm_1/nm_2
    print(indx, match)

    if opts.verbose and match < 0.95:
        P.print_params()

    ###
    ### Create time-domain plots of the two approximants
    ### [util_NRCompareSimulations.py is similar]
    ###
    if opts.save_plots:
        hT_1 = lalsimutils.DataInverseFourier(hF_1)
        hT_2 = lalsimutils.DataInverseFourier(hF_2)
        tvals = lalsimutils.evaluate_tvals(hT_1) - float(P.tref)
        
        # if I have an FD approximant, the times are not necessarily set correctly
        # if lalsim.SimInspiralImplementedFDApproximants(lalsim.GetApproximantFromString(opts.approx))==1:
        #     indx = np.argmax(hT_1.data.data)
        #     indx_0 = int(-tvals[0]/P.deltaT)
        #     print " Rolling by ", indx-indx_0, indx, indx_0
        #     hT_1.data.data = np.roll(hT_1.data.data, -indx+indx_0)
        # if lalsim.SimInspiralImplementedFDApproximants(lalsim.GetApproximantFromString(opts.approx2))==1:
        #     indx = np.argmax(hT_2.data.data)
        #     indx_0 = int(-tvals[0]/P.deltaT)
        #     hT_2.data.data = np.roll(hT_2.data.data, -indx+indx_0)
        #     print " Rolling by ", indx-indx_0,indx,indx_0

        plt.plot(tvals, np.real(hT_1.data.data),label=opts.approx)
        plt.plot(tvals, np.abs(hT_1.data.data),label=opts.approx)
        plt.plot(tvals, np.real(hT_2.data.data),label=opts.approx2)
#        plt.xlim(-1, 0.1)
        plt.legend()
        plt.savefig("post_compare_"+opts.approx+"_"+opts.approx2+"_"+str(indx)+".png")
        plt.clf()



#
# sky_rotations.py
#   Rotate the sky to a specific detector frame pair (usually HL)
#   Note this currently uses GLOBAL STATE VARIABLES for efficiency
#
#   this is *horribly inefficient nonvectorized code*
#   Might want to call astropy or some related library that does this better.\\
#
# NOTE
#   - in practice, we should be rotating the vectors defining the detetcor response etc instead of rotating all our sky
# coordinates every iteration.

import numpy as np
import lalsimulation as lalsim
import lal
import RIFT.lalsimutils as lalsimutils
import functools

# defaults at start
# these global variables don't work as well as we would like
frm=np.diag(np.ones(3))
frmInverse=np.diag(np.ones(3))
vecZ = np.array([0,0,1])
vecZnew = np.array([0,0,1])


def assign_sky_frame(det0,det1,theEpochFiducial):
    global frm
    global frmInverse
    global vecZ
    global vecZnew
    theDetectorSeparation = lalsim.DetectorPrefixToLALDetector(det0).location - lalsim.DetectorPrefixToLALDetector(det1).location
    vecZ = np.array(theDetectorSeparation/np.sqrt( np.dot(theDetectorSeparation,theDetectorSeparation)))
    time_angle =  np.mod( lal.GreenwichMeanSiderealTime(theEpochFiducial), 2*np.pi)
    vecZnew = np.dot(lalsimutils.rotation_matrix(np.array([0,0,1]), -time_angle), vecZ)
    frm = lalsimutils.VectorToFrame(vecZnew)   # Create an orthonormal frame related to this particular choice of z axis. (Used as 'rotation' object)
    frmInverse= np.asarray(np.matrix(frm).I)                                    # Create an orthonormal frame to undo the transform above


# USE INSTEAD
#  functools(lalsimutils.polar_angles_in_frame_alt,frm)
def rotate_sky_forwards_scalar(theta,phi,frm=frm):   # When theta=0 we are describing the coordinats of the zhat direction in the vecZ frame
#    global frm
    return lalsimutils.polar_angles_in_frame_alt(frm, theta,phi)

# superfluous
# USE INSTEAD
#  functools(lalsimutils.polar_angles_in_frame_alt,frm)
def rotate_sky_forwards(th,ph,frm=frm):
    th_out= np.zeros(len(th))
    ph_out= np.zeros(len(th))
    for indx in np.arange(len(th)):
        th_out[indx],ph_out[indx] = rotate_sky_forwards_scalar(th[indx],ph[indx],frm=frm)
        return th_out, ph_out
    

#  functools(lalsimutils.polar_angles_in_frame_alt,frmInverse)
def rotate_sky_backwards_scalar(theta,phi,frmInverse=frmInverse): # When theta=0, the vector should be along the vecZ direction and the polar angles returned its polar angles
#    global frmInverse
    return lalsimutils.polar_angles_in_frame_alt(frmInverse, theta,phi)

#  functools(lalsimutils.polar_angles_in_frame_alt,frmInverse)
def rotate_sky_backwards(th,ph,frmInverse=frmInverse):
    th_out= np.zeros(len(th))
    ph_out= np.zeros(len(th))
    for indx in np.arange(len(th)):
            th_out[indx],ph_out[indx] = rotate_sky_backwards_scalar(th[indx],ph[indx],frmInverse=frmInverse)
    return th_out, ph_out



#! /usr/bin/env python



import os
import sys
import glob
import argparse
#import json
#import bisect
#import re
#from collections import defaultdict
#from argparse import ArgumentParser
#from copy import copy

from matplotlib import pyplot as plt

import scipy.stats

from ligo.lw import utils, ligolw, lsctables # , ilwd
lsctables.use_in(ligolw.LIGOLWContentHandler)
from ligo.lw.utils import process

import h5py
import numpy
np = numpy
from sklearn.neighbors import BallTree


import lal
import lalsimulation
from RIFT.misc import amrlib
from RIFT import lalsimutils

from scipy.special import logsumexp


parser = argparse.ArgumentParser()
parser.add_argument("--fname-hdf",default="intrinsic_grid_all_iterations.hdf")
parser.add_argument("--working-directory",default=".")
parser.add_argument("--test-ks",action='store_true')
opts=  parser.parse_args()

# ass in util_AMRGrid.py.  Should be in 'amrlib'
def get_evidence_grid(points, res_pts, intr_prms, exact=False):
    """
    Associate the "z-axis" value (evidence, overlap, etc...) res_pts with its
    corresponding point in the template bank (points). If exact is True, then
    the poit must exactly match the point in the bank.
    """
    grid_tree = BallTree(selected)
    grid_idx = []
    # Reorder the grid points to match their weight indices
    for res in res_pts:
        dist, idx = grid_tree.query(numpy.atleast_2d(res), k=1)
        # Stupid floating point inexactitude...
        #print res, selected[idx[0][0]]
        #assert numpy.allclose(res, selected[idx[0][0]])
        grid_idx.append(idx[0][0])
    return points[grid_idx]

def load_composite(base, indx):
    fname_composite = base + "/consolidated_{}.composite".format(indx)
    use_composite=True
    tempfile = "temp_convert_file"
    cmd = "convert_output_format_allnet2xml --fname {} --fname-output-samples temp_convert_file ".format(fname_composite)
    os.system(cmd)
    fname_composite = "temp_convert_file.xml.gz"
    results = []
    xmldoc = utils.load_filename(fname_composite, contenthandler=ligolw.LIGOLWContentHandler)
    results.extend(lsctables.SimInspiralTable.get_table(xmldoc))
    # Space grid
    res_pts = numpy.array([tuple(getattr(t, a) for a in intr_prms) for t in results]) # can fill spin components if present

    # log Evidence values
#    maxlnevid = numpy.max([s.alpha3 for s in results])
#    total_evid = numpy.exp([s.alpha3 - maxlnevid for s in results]).sum()
#    for res in results:
#        res.alpha3 = numpy.exp(res.alpha3 - maxlnevid)/total_evid  # (a) convert lnL -> exp(lnL), but also rescale
    results = numpy.array([res.alpha3 for res in results])  # convert to scalar array

    return res_pts, results   # results are log evidences, with correct normalization


# Load AMR


# currently hardcoded here, will make more flexible
intr_prms = ["mchirp","eta", "spin1z", "spin2z"]




# Load results file
#  - in principle, this has everything we need, IF results are already gridded.
# fname_composite = "all.net"
# if '.composite' in fname_composite  or '.net' in fname_composite:
#         use_composite=True
#         tempfile = "temp_convert_file"
#         cmd = "convert_output_format_allnet2xml --fname {} --fname-output-samples temp_convert_file ".format(fname_composite)
#         os.system(cmd)
#         fname_composite = "temp_convert_file.xml.gz"
# results = []
# xmldoc = utils.load_filename(fname_composite, contenthandler=ligolw.LIGOLWContentHandler)
# results.extend(lsctables.SimInspiralTable.get_table(xmldoc))

fname_hdf = opts.fname_hdf # "intrinsic_grid_all_iterations.hdf"


# count the number of composite files
fnames_composite = list(glob.glob(opts.working_directory+"/consolidated_*.composite"))

log_Zl = []
results_saved=[]
for indx in range(0,len(fnames_composite)):
    # Note this only works with a *complete* sample at every level .. if I start truncating, it fails
    # Evidence values
    res_pts, results = load_composite(opts.working_directory,indx)

    dim = 4  # default; effective dimension count
#    print(res_pts[:10], results[:10])

    # this is dumb, we can just get the spacing from top level
    amr_current_level = indx
    (prev_cells, spacing), level, _ = amrlib.load_grid_level(fname_hdf, amr_current_level, True)


    # Perform Riemann sum
    #   - assumes we are all ok
    #    print(spacing)
    dv = np.prod(spacing)
    log_Zl_here = logsumexp(results) + np.log(dv)
    log_Zl.append(log_Zl_here)

    if not(opts.test_ks):
        print(indx, log_Zl_here)
    else:
        # 1d analysis
        #  - nearest neihbor spacing
        vals_sorted = results[np.argsort(results)]
        mean_diff = np.mean(np.diff(vals_sorted))  # which is just of course delta X/N, performing the sum
    
        lnLmax = np.max(vals_sorted)
        
        def my_cdf(lnL):
            return 1.-scipy.stats.chi2.cdf((lnLmax-lnL)*2,df=dim)

        val_ks = scipy.stats.kstest(vals_sorted, my_cdf)

        plt.clf()
        plt.scatter( vals_sorted , np.arange(len(vals_sorted))/(1.0*len(vals_sorted)))
        plt.xlabel(r'$w=2(\ln L_{\rm max} - \ln L)$')
        plt.ylabel(r'$P(<w)$')
        plt.plot( vals_sorted, my_cdf(vals_sorted))
        plt.savefig("my_fig_{}.pdf".format(indx))
    

        print(indx, log_Zl_here, mean_diff, val_ks)


# Convergence log (only works if not throttled)
print(log_Zl)




## EXTRA CODE
def superfluous_operations():
    # Load hdf5 file
    #  - copied verbatim from util_AMRGrid.py
    init_region, region_labels = amrlib.load_init_region(fname_hdf, get_labels=True)
    reindex = numpy.array([list(region_labels).index(l) for l in intr_prms])
    intr_prms = list(region_labels)

    # Remap/reorder loaded grid points to have desired parameter name order
    res_pts = res_pts[:,reindex]
    # now overwrite eta values (all filled with 0 since no attr) with correct value
    eta_indx = intr_prms.index('eta')
    mass_pts = numpy.array([tuple(getattr(t, a) for a in ['mass1','mass2']) for t in results])
    res_pts[:,eta_indx] = lalsimutils.symRatio(mass_pts[:,0],mass_pts[:,1])

    # Raw coordinates as currently constructed
    print(res_pts[:10])




